@@ -2,12 +2,22 @@ import numpy as np
 import radvel.model
 from radvel import gp
 from scipy.linalg import cho_factor, cho_solve
-from scipy import matrix
+import warnings
+
 
 _has_celerite = gp._try_celerite()
 if _has_celerite:
     import celerite
 
+
+def custom_formatwarning(msg, *args, **kwargs):
+    # ignore everything except the message
+    return str(msg) + '\n'
+
+
+warnings.formatwarning = custom_formatwarning
+
+
 class Likelihood(object):
     """
     Generic Likelihood
@@ -26,6 +36,8 @@ class Likelihood(object):
             self.params[key] = radvel.model.Parameter(value=0.0)
         self.uparams = None
 
+        self.params_order = self.list_vary_params()
+
     def __repr__(self):
         s = ""
         if self.uparams is None:
@@ -34,9 +46,9 @@ class Likelihood(object):
                 )
             keys = self.params.keys()
             for key in keys:
-                vstr = str(self.params[key].vary)    
+                vstr = str(self.params[key].vary)
                 if (key.startswith('tc') or key.startswith('tp')) and self.params[key].value > 1e6:
-                    par = self.params[key].value - 2450000
+                    par = self.params[key].value - self.model.time_base
                 else:
                     par = self.params[key].value
 
@@ -57,21 +69,19 @@ class Likelihood(object):
                     err = 0
                 if (key.startswith('tc') or key.startswith('tp')) and \
                         self.params[key].value > 1e6:
-                    par = self.params[key].value - 2450000
+                    par = self.params[key].value - self.model.time_base
                 else:
                     par = self.params[key].value
-                    
+
                 s += "{:20s}{:15g}{:10g}{:>10s}\n".format(
                     key, par, err, vstr
                      )
         return s
 
     def set_vary_params(self, param_values_array):
+        param_values_array = list(param_values_array)
         i = 0
         for key in self.list_vary_params():
-            # flip sign for negative jitter
-            if key.startswith('jit') and param_values_array[i] < 0:
-                param_values_array[i] = -param_values_array[i]
             self.params[key].value = param_values_array[i]
             i += 1
         assert i == len(param_values_array), \
@@ -81,12 +91,22 @@ class Likelihood(object):
         params_array = []
         for key in self.list_vary_params():
             if self.params[key].vary:
-                params_array += [self.params[key].value]     
+                params_array += [self.params[key].value]
         params_array = np.array(params_array)
         return params_array
 
     def list_vary_params(self):
-        return [key for key in self.params.keys() if self.params[key].vary]
+        keys = self.list_params()
+
+        return [key for key in keys if self.params[key].vary]
+
+    def list_params(self):
+        try:
+            keys = self.params_order
+        except AttributeError:
+            keys = list(self.params.keys())
+            self.params_order = keys
+        return keys
 
     def residuals(self):
         return self.y - self.model(self.x)
@@ -102,6 +122,44 @@ class Likelihood(object):
         _logprob = self.logprob()
         return _logprob
 
+    def bic(self):
+        """
+        Calculate the Bayesian information criterion
+
+        Returns:
+            float: BIC
+        """
+
+        n = len(self.y)
+        k = len(self.get_vary_params())
+        _bic = np.log(n) * k - 2.0 * self.logprob()
+        return _bic
+
+    def aic(self):
+        """
+        Calculate the Aikike information criterion
+        The Small Sample AIC (AICC) is returned because for most RV data sets n < 40 * k
+        (see Burnham & Anderson 2002 S2.4).
+
+        Returns:
+            float: AICC
+        """
+
+        n = len(self.y)
+        k = len(self.get_vary_params())
+        aic = - 2.0 * self.logprob() + 2.0 * k
+        # Small sample correction
+        _aicc = aic
+        denom = (n - k - 1.0)
+        if denom > 0:
+            _aicc += (2.0 * k * (k + 1.0)) / denom
+        else:
+            print("Warning: The number of free parameters is greater than or equal to")
+            print("         the number of data points. The AICc comparison calculations")
+            print("         will fail in this case.")
+            _aicc = np.inf
+        return _aicc
+
 
 class CompositeLikelihood(Likelihood):
     """Composite Likelihood
@@ -120,27 +178,30 @@ class CompositeLikelihood(Likelihood):
         params = like0.params
         self.model = like0.model
         self.x = like0.x
-        self.y = like0.y #- params[like0.gamma_param].value
+        self.y = like0.y
         self.yerr = like0.yerr
         self.telvec = like0.telvec
         self.extra_params = like0.extra_params
         self.suffixes = like0.suffix
         self.uparams = like0.uparams
-        
+        self.hnames = []
+
         for i in range(1, self.nlike):
             like = like_list[i]
-            
+
             self.x = np.append(self.x, like.x)
             self.y = np.append(self.y, like.y - like.params[like.gamma_param].value)
             self.yerr = np.append(self.yerr, like.yerr)
             self.telvec = np.append(self.telvec, like.telvec)
             self.extra_params = np.append(self.extra_params, like.extra_params)
             self.suffixes = np.append(self.suffixes, like.suffix)
+            if hasattr(like, 'hnames'):
+                self.hnames.extend(like.hnames)
             try:
                 self.uparams = self.uparams.update(like.uparams)
             except AttributeError:
                 self.uparams = None
-            
+
             assert like.model is like0.model, \
                 "Likelihoods must use the same model"
 
@@ -153,7 +214,7 @@ class CompositeLikelihood(Likelihood):
         self.extra_params = list(set(self.extra_params))
         self.params = params
         self.like_list = like_list
-        
+
     def logprob(self):
         """
         See `radvel.likelihood.RVLikelihood.logprob`
@@ -173,7 +234,7 @@ class CompositeLikelihood(Likelihood):
             res = np.append(res, like.residuals())
 
         return res
-        
+
     def errorbars(self):
         """
         See `radvel.likelihood.RVLikelihood.errorbars`
@@ -184,6 +245,7 @@ class CompositeLikelihood(Likelihood):
 
         return err
 
+
 class RVLikelihood(Likelihood):
     """RV Likelihood
 
@@ -210,7 +272,7 @@ class RVLikelihood(Likelihood):
             self.suffix = suffix
 
         self.telvec = np.array([self.suffix]*len(t))
-        
+
         self.decorr_params = []
         self.decorr_vectors = decorr_vectors
         if len(decorr_vars) > 0:
@@ -226,8 +288,17 @@ class RVLikelihood(Likelihood):
 
         Data minus model
         """
-        res = self.y - self.params[self.gamma_param].value - self.model(self.x)
-        
+        mod = self.model(self.x)
+
+        if self.params[self.gamma_param].linear and not self.params[self.gamma_param].vary:
+            ztil = np.sum((self.y - mod)/(self.yerr**2 + self.params[self.jit_param].value**2)) / \
+                   np.sum(1/(self.yerr**2 + self.params[self.jit_param].value**2))
+            if np.isnan(ztil):
+                 ztil = 0.0
+            self.params[self.gamma_param].value = ztil
+
+        res = self.y - self.params[self.gamma_param].value - mod
+
         if len(self.decorr_params) > 0:
             for parname in self.decorr_params:
                 var = parname.split('_')[1]
@@ -249,7 +320,7 @@ class RVLikelihood(Likelihood):
 
         Returns:
             array: uncertainties
-        
+
         """
         return np.sqrt(self.yerr**2 + self.params[self.jit_param].value**2)
 
@@ -261,11 +332,15 @@ class RVLikelihood(Likelihood):
         Returns:
             float: Natural log of likelihood
         """
-        
+
         sigma_jit = self.params[self.jit_param].value
         residuals = self.residuals()
         loglike = loglike_jitter(residuals, self.yerr, sigma_jit)
-        
+
+        if self.params[self.gamma_param].linear and not self.params[self.gamma_param].vary:
+            sigz = 1/np.sum(1 / (self.yerr**2 + sigma_jit**2))
+            loglike += np.log(np.sqrt(2 * np.pi * sigz))
+
         return loglike
 
 
@@ -279,18 +354,18 @@ class GPLikelihood(RVLikelihood):
         t (array): time array
         vel (array): array of velocities
         errvel (array): array of velocity uncertainties
-        hnames (list of string): keys corresponding to radvel.Parameter 
+        hnames (list of string): keys corresponding to radvel.Parameter
            objects in model.params that are GP hyperparameters
         suffix (string): suffix to identify this Likelihood object;
            useful when constructing a `CompositeLikelihood` object
     """
-    def __init__(self, model, t, vel, errvel, 
+    def __init__(self, model, t, vel, errvel,
                  hnames=['gp_per', 'gp_perlength', 'gp_explength', 'gp_amp'],
                  suffix='', kernel_name="QuasiPer", **kwargs):
 
         self.suffix = suffix
         super(GPLikelihood, self).__init__(
-              model, t, vel, errvel, suffix=self.suffix, 
+              model, t, vel, errvel, suffix=self.suffix,
               decorr_vars=[], decorr_vectors={}
             )
         assert kernel_name in gp.KERNELS.keys(), \
@@ -300,17 +375,17 @@ class GPLikelihood(RVLikelihood):
         self.hnames = hnames  # list of string names of hyperparameters
         self.hyperparams = {k: self.params[k] for k in self.hnames}
 
-        self.kernel_call = getattr(gp, kernel_name + "Kernel") 
+        self.kernel_call = getattr(gp, kernel_name + "Kernel")
         self.kernel = self.kernel_call(self.hyperparams)
 
         self.kernel.compute_distances(self.x, self.x)
         self.N = len(self.x)
 
     def update_kernel_params(self):
-        """ Update the Kernel object with new values of the hyperparameters 
+        """ Update the Kernel object with new values of the hyperparameters
         """
         for key in self.list_vary_params():
-            if key in self.hnames: 
+            if key in self.hnames:
                 hparams_key = key.split('_')
                 hparams_key = hparams_key[0] + '_' + hparams_key[1]
                 self.kernel.hparams[hparams_key].value = self.params[key].value
@@ -341,11 +416,11 @@ class GPLikelihood(RVLikelihood):
         .. math::
 
            lnL = -0.5r^TK^{-1}r - 0.5ln[det(K)] - 0.5N*ln(2pi)
-           
-        where r = vector of residuals (GPLikelihood._resids), 
-        K = covariance matrix, and N = number of datapoints. 
 
-        Priors are not applied here. 
+        where r = vector of residuals (GPLikelihood._resids),
+        K = covariance matrix, and N = number of datapoints.
+
+        Priors are not applied here.
         Constant has been omitted.
 
         Returns:
@@ -355,7 +430,7 @@ class GPLikelihood(RVLikelihood):
         # update the Kernel object hyperparameter values
         self.update_kernel_params()
 
-        r = self._resids()      
+        r = self._resids()
 
         self.kernel.compute_covmatrix(self.errorbars())
 
@@ -373,10 +448,9 @@ class GPLikelihood(RVLikelihood):
 
             return like
 
-        except np.linalg.linalg.LinAlgError:
-            print("Warning: non-positive definite kernel detected.")
-            return -np.inf 
-
+        except (np.linalg.linalg.LinAlgError, ValueError):
+            warnings.warn("Non-positive definite kernel detected.", RuntimeWarning)
+            return -np.inf
 
     def predict(self, xpred):
         """ Realize the GP using the current values of the hyperparameters at values x=xpred.
@@ -392,7 +466,7 @@ class GPLikelihood(RVLikelihood):
 
         self.update_kernel_params()
 
-        r = matrix(self._resids()).T
+        r = np.array([self._resids()]).T
 
         self.kernel.compute_distances(self.x, self.x)
         K = self.kernel.compute_covmatrix(self.errorbars())
@@ -402,27 +476,28 @@ class GPLikelihood(RVLikelihood):
 
         L = cho_factor(K)
         alpha = cho_solve(L, r)
-        mu = np.array(Ks*alpha).flatten()
+        mu = np.dot(Ks, alpha).flatten()
 
         self.kernel.compute_distances(xpred, xpred)
         Kss = self.kernel.compute_covmatrix(0.)
-        B = cho_solve(L, Ks.T) 
-        var = np.array(np.diag(Kss - Ks * matrix(B))).flatten()
+        B = cho_solve(L, Ks.T)
+        var = np.array(np.diag(Kss - np.dot(Ks, B))).flatten()
         stdev = np.sqrt(var)
 
         # set the default distances back to their regular values
         self.kernel.compute_distances(self.x, self.x)
 
         return mu, stdev
-        
+
 
 class CeleriteLikelihood(GPLikelihood):
     """Celerite GP Likelihood
 
     The Likelihood object for a radial velocity dataset modeled with a GP
-    whose kernel is a sum of celerite terms. 
+    whose kernel is an approximation to the quasi-periodic kernel.
+
     See celerite.readthedocs.io and Foreman-Mackey et al. 2017. AJ, 154, 220
-    for more details.
+    (equation 56) for more details.
 
     See `radvel/example_planets/k2-131_celerite.py` for an example of a setup
     file that uses this Likelihood object.
@@ -432,71 +507,45 @@ class CeleriteLikelihood(GPLikelihood):
         t (array): time array
         vel (array): array of velocities
         errvel (array): array of velocity uncertainties
-        hnames (list of string): keys corresponding to radvel.Parameter 
+        hnames (list of string): keys corresponding to radvel.Parameter
            objects in model.params that are GP hyperparameters
         suffix (string): suffix to identify this Likelihood object;
            useful when constructing a `CompositeLikelihood` object
     """
 
     def __init__(self, model, t, vel, errvel, hnames, suffix='', **kwargs):
+
         super(CeleriteLikelihood, self).__init__(
             model, t, vel, errvel, hnames,
             suffix=suffix, kernel_name='Celerite'
         )
 
-        # sort inputs in time order. required for celerite.
+        # Sort inputs in time order. Required for celerite calculations.
         order = np.argsort(self.x)
         self.x = self.x[order]
         self.y = self.y[order]
         self.yerr = self.yerr[order]
         self.N = len(self.x)
 
-    def update_kernel_params(self):
-        """ Update the Kernel object with new values of the hyperparameters 
-        """
-        for key in self.list_vary_params():
-            if key in self.hnames:  
-                index = int(key[0][-1]) - 1 # grab index from hyperparameter string name
-                if 'logA' in key:
-                    self.kernel.hparams[index,0] = self.params[key].value
-                if 'logB' in key:
-                    self.kernel.hparams[index,1] = self.params[key].value
-                if 'logC' in key:
-                    self.kernel.hparams[index,2] = self.params[key].value
-                if 'logD' in key:
-                    self.kernel.hparams[index,3] = self.params[key].value
-
     def logprob(self):
 
         self.update_kernel_params()
 
-        can_proceed = True
-        for i in np.arange(self.kernel.num_terms):
-            if np.exp(self.kernel.hparams[i,0]+self.kernel.hparams[i,2]) >= \
-               np.exp(self.kernel.hparams[i,1]+self.kernel.hparams[i,3]):
-                pass
-            else:
-                can_proceed = False
-
-        if can_proceed:
-            r = self._resids() 
-
+        try:
             solver = self.kernel.compute_covmatrix(self.errorbars())
 
             # calculate log likelihood
-            lnlike =  -0.5 * (solver.dot_solve(self._resids()) + solver.log_determinant() + self.N*np.log(2.*np.pi))
-        
+            lnlike = -0.5 * (solver.dot_solve(self._resids()) + solver.log_determinant() + self.N*np.log(2.*np.pi))
+
             return lnlike
 
-        else:
-           print("WARNING: CeleriteKernel has encountered a non-positive-definite"
-                +" kernel. Ensure that ac>=bd for all complex kernel terms.")
-
-           return -np.inf
+        except celerite.solver.LinAlgError:
+            warnings.warn("Non-positive definite kernel detected.", RuntimeWarning)
+            return -np.inf
 
     def predict(self,xpred):
         """ Realize the GP using the current values of the hyperparameters at values x=xpred.
-            Used for making GP plots. Wrapper around `celerite.GP.predict()`.
+            Used for making GP plots. Wrapper for `celerite.GP.predict()`.
 
             Args:
                 xpred (np.array): numpy array of x values for realizing the GP
@@ -508,20 +557,31 @@ class CeleriteLikelihood(GPLikelihood):
 
         self.update_kernel_params()
 
+        B = self.kernel.hparams['gp_B'].value
+        C = self.kernel.hparams['gp_C'].value
+        L = self.kernel.hparams['gp_L'].value
+        Prot = self.kernel.hparams['gp_Prot'].value
+
         # build celerite kernel with current values of hparams
         kernel = celerite.terms.JitterTerm(
                 log_sigma = np.log(self.params[self.jit_param].value)
                 )
-        for i in np.arange(self.kernel.num_terms):
-            kernel = kernel + celerite.terms.ComplexTerm(
-                log_a = self.kernel.hparams[i,0],
-                log_b = self.kernel.hparams[i,1],
-                log_c = self.kernel.hparams[i,2],
-                log_d = self.kernel.hparams[i,3]
-                )
+
+        kernel += celerite.terms.RealTerm(
+            log_a=np.log(B*(1+C)/(2+C)),
+            log_c=np.log(1/L)
+        )
+
+        kernel += celerite.terms.ComplexTerm(
+            log_a=np.log(B/(2+C)),
+            log_b=-np.inf,
+            log_c=np.log(1/L),
+            log_d=np.log(2*np.pi/Prot)
+        )
+
         gp = celerite.GP(kernel)
         gp.compute(self.x, self.yerr)
-        mu, var = gp.predict(self.y-self.params[self.gamma_param].value, xpred, return_var=True)
+        mu, var = gp.predict(self._resids(), xpred, return_var=True)
 
         stdev = np.sqrt(var)
 
@@ -532,10 +592,10 @@ def loglike_jitter(residuals, sigma, sigma_jit):
     """
     Log-likelihood incorporating jitter
 
-    See equation (1) in Howard et al. 2014. Returns loglikelihood, where 
+    See equation (1) in Howard et al. 2014. Returns loglikelihood, where
     sigma**2 is replaced by sigma**2 + sigma_jit**2. It penalizes
     excessively large values of jitter
-    
+
     Args:
         residuals (array): array of residuals
         sigma (array): array of measurement errors
@@ -548,5 +608,5 @@ def loglike_jitter(residuals, sigma, sigma_jit):
     penalty = np.sum( np.log( np.sqrt( 2 * np.pi * sum_sig_quad ) ) )
     chi2 = np.sum(residuals**2 / sum_sig_quad)
     loglike = -0.5 * chi2 - penalty
-    
+
     return loglike
